@@ -97,6 +97,18 @@ func client(config *clientConfig) {
 	if len(config.Obfs) > 0 {
 		obfuscator = obfs.NewXPlusObfuscator([]byte(config.Obfs))
 	}
+	// Resolve preference
+	if len(config.ResolvePreference) > 0 {
+		pref, excl, err := transport.ResolvePreferenceFromString(config.ResolvePreference)
+		if err != nil {
+			logrus.WithFields(logrus.Fields{
+				"error": err,
+			}).Fatal("Failed to parse the resolve preference")
+		}
+		transport.DefaultClientTransport.PrefEnabled = true
+		transport.DefaultClientTransport.PrefIPv6 = pref
+		transport.DefaultClientTransport.PrefExclusive = excl
+	}
 	// ACL
 	var aclEngine *acl.Engine
 	if len(config.ACL) > 0 {
@@ -119,10 +131,11 @@ func client(config *clientConfig) {
 	// Client
 	var client *core.Client
 	try := 0
+	up, down, _ := config.Speed()
 	for {
 		try += 1
 		c, err := core.NewClient(config.Server, config.Protocol, auth, tlsConfig, quicConfig,
-			transport.DefaultClientTransport, uint64(config.UpMbps)*mbpsToBps, uint64(config.DownMbps)*mbpsToBps,
+			transport.DefaultClientTransport, up, down,
 			func(refBPS uint64) congestion.CongestionControl {
 				return hyCongestion.NewBrutalSender(congestion.ByteCount(refBPS))
 			}, obfuscator)
